/*****************************************************************************
 * file    lib/process/main.c  
 * author  YuLiang
 * version 1.0.0
 * date    26-Sep-2021
 * brief   This file provides all the main related operation functions.
 ******************************************************************************
 * Attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2021 LandPower</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of LandPower nor the names of its contributors may be used to 
 *      endorse or promote products derived from this software without specific
 *      prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* 标准C库头文件. */
#include <locale.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/rtc.h>
#include <time.h>

#include "cmd.h"
#include "mtimer.h"
#include "main.h"
#include "hwgpio.h"
#include "fifo.h"
#include "pd_main.h"

/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
int32_t recv_qid;
uint16_t version_hex;
uint32_t start_time;

static int8_t is_system_reboot;

static reboot_msg_t _reboot_msg[] = 
{
    {REBOOT_NONE, "Reboot none."},
    {REBOOT_LOCAL_SERVER_IP_CHANGE, "Reboot by debug tool has changed server."},
    {REBOOT_LOCAL_IP_CHANGE, "Reboot by debug tool has changed device ip."},
    {REBOOT_LOCAL_HOST_NAME_CHANGE, "Reboot by debug tool has changed device id."},
    {REBOOT_LOCAL_RESET, "Reboot by debug tool reset."},
    {REBOOT_LOCAL_ARM_UPGRADE, "Reboot by debug tool softwave upgrade."},
    {REBOOT_REMOTE_SERVER_IP_CHANGE, "Reboot by remote has changed server."},
    {REBOOT_REMOTE_IP_CHANGE, "Reboot by remote has changed device ip."},
    {REBOOT_REMOTE_HOST_NAME_CHANGE, "Reboot by remote has changed device id."},
    {REBOOT_REMOTE_RESET, "Reboot by remote reset."},
    {REBOOT_UPGRADE_ALL, "Reboot by softwave upgrade."},
    {REBOOT_SYSTEM_RESET, "Reboot by command."},
    {REBOOT_4G_ERROR, "Reboot by 4G error."},
    {REBOOT_NET_ERROR, "Reboot by CSG platform connection time out."},
    {REBOOT_MAX, NULL}
};

/* Private function prototypes -----------------------------------------------*/

/* Internal functions --------------------------------------------------------*/
/* 信号处理函数 */
void _signal_handler(int sig)
{
    if (SIGSEGV == sig)
    {
        log_backtrace(LOG_LVL_ERR);
    }
    //else if(SIGINT == sig
    //    || SIGTSTP == sig)
    //{
        /* 屏蔽信号 */
    //    return;
    //}

    exit(-1);
}

/* description: 设备重启函数
   param: module -- 记录在 哪个log 模块
          type   -- 重启原因
   return: */
void reboot_system(int module, REBOOT_MSG type)
{
    char *pmsg = NULL;

    is_system_reboot = TRUE;
    
    for (REBOOT_MSG i = REBOOT_NONE; i < REBOOT_MAX; i++)
    {
        if (_reboot_msg[i].type == type)
        {
            pmsg = _reboot_msg[i].msg;
            break;
        }
    }
    
    if (pmsg)
    {
        log_out(module, LOG_LVL_WARN, "%s", pmsg);
        system("sync");
    }

    sleep(3);
    system("reboot -f");
}

/* description: 数据处理预初始化.
   param:
   return:  (E_NONE)成功,(其他)失败 */
int32_t process_init(void)
{
    int32_t rv = E_NONE;

    /* 创建消息队列. */
    if ((recv_qid = msgget(0x4321, IPC_CREAT | 0666)) == -1)
    {
        log_err(LOG_DEFAULT, "message ERROR at msgget return %s!", safe_strerror(errno));
    }
    
    /* 清空消息队列. */
    msgctl(recv_qid, IPC_RMID, NULL);
    if ((recv_qid = msgget(0x4321, IPC_CREAT | 0666)) == -1)
    {
        log_err(LOG_DEFAULT, "message ERROR at msgget return %s!", safe_strerror(errno));
    }

    /* 初始化局放应用. */
    rv |= pd_main();

    return rv;
}

/* description: 数据处理初始化.
   param:
   return:  (E_NONE)成功,(其他)失败 */
int32_t process_init_after(void)
{
    int32_t rv = E_NONE;

    rv = pd_main_after();

    return rv;
}

/* description: 程序入口函数.
   param:
   return: */
int32_t main(int32_t argc, char **argv)
{
    struct sigaction act;
    uint32_t cnt = 0;
    
    /* 设置本地化信息为默认值. */
    setlocale(LC_ALL, "");

    /* 必须最前面, 初始化内存管理模块的基本参数. */
    mtype_init_befor();

    /* log初始化 */
    log_open();
    
    log_out(LOG_DEFAULT, LOG_LVL_WARN, "System start!");
    
    /* 设置信号处理的回调函数 */
    act.sa_handler = _signal_handler;
    sigemptyset(&act.sa_mask);
    sigaction(SIGINT, &act, NULL);
    sigaction(SIGSEGV, &act, NULL);
    sigaction(SIGTSTP, &act, NULL);
    
    /* 初始化cli等公共模块. */
    cmd_init();
    thread_m_init();
    mtype_init();
    dbg_init();
    mtimer_init();
    vtysh_init();
    vtycmd_init();
    gpio_init();
    fifo_init();
    log_handle_init();

    /* 主处理函数预初始化 */
    process_init();
    
    /* 配置恢复命令行启动 */
    vtysh_config_recovery();

    /* 主处理函数初始化 */
    process_init_after();

    /* 启动命令行 */
    vtysh_shell_init();
    vtycmd_cmd_init();

    /* 初始化完成 */
    version_hex = version_str_to_int();
    is_system_init = TRUE;
    
    /* 主循环, 点灯喂狗. */
    for(;;)
    {
        sleep(1);
        start_time++;
        cnt++;
    }
    
    return 0;
}
/************************ (C) COPYRIGHT LandPower ***** END OF FILE ****/
